/*
** Module   : EditorInterface.java
** Abstract : Define all EDITOR-specific widget attributes and methods available via a handle 
**            instance.
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20141120 Created initial version.
** 002 EVK 20150209 Added conversion support for SAVE-FILE method.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Define all EDITOR-specific widget attributes and methods which can be accessed via a handle
 * instance.  Attributes and methods shared by multiple widget types will be defined elsewhere.
 */
public interface EditorInterface
{
   /**
    * Implements the DELETE-LINE() method which deletes the line of text in which the cursor is
    * positioned.
    *
    * @return   <code>true</code> if the line was deleted.
    */
   @LegacyMethod(name = "DELETE-LINE")
   public logical deleteLine();
   
   /**
    * Implements the INSERT-STRING() method which inserts the given text into the editor at the
    * location at which the cursor is positioned.
    *
    * @param    txt
    *           The text to be inserted.
    *
    * @return   <code>true</code> if the text was inserted.
    */
   @LegacyMethod(name = "INSERT-STRING")
   public logical insertString(character txt);
   
   /**
    * Implements the INSERT-STRING() method which inserts the given text into the editor at the
    * location at which the cursor is positioned.
    *
    * @param    txt
    *           The text to be inserted.
    *
    * @return   <code>true</code> if the text was inserted.
    */
   @LegacyMethod(name = "INSERT-STRING")
   public logical insertString(String txt);
   
   /**
    * Implements the READ-FILE() method which replaces the editor's contents with those read
    * from the specified file.
    *
    * @param    name
    *           The relative or absolute file name.
    *
    * @return   <code>true</code> if the file was successfully read.
    */
   @LegacyMethod(name = "READ-FILE")
   public logical readFile(character name);
   
   /**
    * Implements the READ-FILE() method which replaces the editor's contents with those read
    * from the specified file.
    *
    * @param    name
    *           The relative or absolute file name.
    *
    * @return   <code>true</code> if the file was successfully read.
    */
   @LegacyMethod(name = "READ-FILE")
   public logical readFile(String name);
   
   /**
    * Implements the REPLACE-SELECTION-TEXT() method which replaces the editor's currently
    * selected text with the given text.
    *
    * @param    txt
    *           The replacement text.
    *
    * @return   <code>true</code> if the text was successfully replaced.
    */
   @LegacyMethod(name = "REPLACE-SELECTION-TEXT")
   public logical replaceSelectionText(character txt);
   
   /**
    * Implements the REPLACE-SELECTION-TEXT() method which replaces the editor's currently
    * selected text with the given text.
    *
    * @param    txt
    *           The replacement text.
    *
    * @return   <code>true</code> if the text was successfully replaced.
    */
   @LegacyMethod(name = "REPLACE-SELECTION-TEXT")
   public logical replaceSelectionText(String txt);

   /**
    * Implements the SAVE-FILE() method which write the editor's contents to the specified file.
    *
    * @param    name
    *           The relative or absolute file name.
    *
    * @return   <code>true</code> if the file was successfully written.
    */
   @LegacyMethod(name = "SAVE-FILE")
   public logical saveFile(String name);

   /**
    * Implements the SAVE-FILE() method which write the editor's contents to the specified file.
    *
    * @param    name
    *           The relative or absolute file name.
    *
    * @return   <code>true</code> if the file was successfully written.
    */
   @LegacyMethod(name = "SAVE-FILE")
   public logical saveFile(character name);   
   
   /**
    * Get the value of the CURSOR-CHAR attribute which is the current cursor position as a
    * 1-based index on the current line.
    *
    * @return   The current position of the cursor on the current line.
    */
   @LegacyAttribute(name = "CURSOR-CHAR")
   public integer getCursorChar();
   
   /**
    * Set the value of the CURSOR-CHAR attribute which moves the cursor position to the
    * specified 1-based index on the current line.
    *
    * @param    idx
    *           The position to which to move.
    */
   @LegacyAttribute(name = "CURSOR-CHAR", setter = true)
   public void setCursorChar(double idx);

   /**
    * Set the value of the CURSOR-CHAR attribute which moves the cursor position to the
    * specified 1-based index on the current line.
    *
    * @param    idx
    *           The position to which to move.
    */
   @LegacyAttribute(name = "CURSOR-CHAR", setter = true)
   public void setCursorChar(NumberType idx);

   /**
    * Get the value of the CURSOR-LINE attribute which is the current cursor position as a
    * 1-based line number.
    *
    * @return   The current position of the cursor on the current line.
    */
   @LegacyAttribute(name = "CURSOR-LINE")
   public integer getCursorLine();
   
   /**
    * Set the value of the CURSOR-LINE attribute which moves the cursor position to the
    * specified 1-based line number.
    *
    * @param    line
    *           The line to which to move.
    */
   @LegacyAttribute(name = "CURSOR-LINE", setter = true)
   public void setCursorLine(double line);

   /**
    * Set the value of the CURSOR-LINE attribute which moves the cursor position to the
    * specified 1-based line number.
    *
    * @param    line
    *           The line to which to move.
    */
   @LegacyAttribute(name = "CURSOR-LINE", setter = true)
   public void setCursorLine(NumberType line);
   
   /**
    * Get the value of the NUM-LINES attribute which is the number of lines of text in the
    * editor.
    *
    * @return   The number of lines in the editor.
    */
   @LegacyAttribute(name = "NUM-LINES")
   public integer getNumLines();
}
