/*
** Module   : EditorWidget.java
** Abstract : server side Editor widget implementation
**
** Copyright (c) 2006-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ---------------------------Description----------------------------
** 001 SIY 20060427   @26084 Server side part of the EDITOR widget.
** 002 GES 20060803   @28373 Cleanup and refactoring.
** 003 GES 20080320   @37585 Added TYPE attribute.
** 004 SIY 20111101          Updated imports.
** 005 CA  20130129          Added setLarge stub, for EDITOR LARGE.
** 006 CA  20130221          Refactored usage of TYPE attribute.
** 007 CA  20130222          Added NPE protection for cases when a widget is used before is 
**                           attached to a frame (after a CREATE WIDGET statement is used).
** 008 CA  20130927          Added LegacyResource annotation, to determine resource type.
** 009 CA  20131115          Changes related to properly instantiate dynamic resources.
** 010 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 011 SVL 20141102          Make this class implement ScrollbarVerticalElement.
** 012 GES 20141124          Added EditorInterface support (with all associated attrs/meths).
** 013 GES 20150103          Added runtime support for new attrs/methods.
** 014 EVK 20150209          Added runtime support for save-file and read-file methods.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Server side implementation of the EDITOR widget.
 */
@LegacyResource(resource = LegacyResource.EDITOR)
public class EditorWidget
extends ControlTextWidget<EditorConfig>
implements EditorInterface,
           ScrollbarVerticalElement,
           ScrollbarHorizontalElement,
           InnerLines
{
   /**
    * Default constructor.
    */
   public EditorWidget()
   {
      this(false);
   }
   
   /**
    * Default constructor.
    * 
    * @param   dynamic
    *          Flag indicating if this is a static or dynamic resource.
    */
   public EditorWidget(boolean dynamic)
   {
      super(dynamic, new EditorConfig());
   }

   /**
    * Mark this EDITOR widget as EDITOR LARGE, which can be used to display a value of type 
    * <code>LONGCHAR</code>.
    * 
    * @param    large
    *           When <code>true</code>, this is a EDITOR LARGE widget.
    */
   public void setLarge(boolean large)
   {
      UnimplementedFeature.missing("setLarge() method");
   }
   
   /**
    * Gets the value of the INNER-LINES attribute which defines the number of text lines
    * in the widget.
    * 
    * @return   The number of lines of text.
    */
   public integer getInnerLines()
   {
      return new integer(config.innerLines);
   }
   
   /**
    * Sets the value of the INNER-LINES attribute which defines the number of text lines
    * in the widget.
    * 
    * @param    lines
    *           The new number of lines.
    */
   public void setInnerLines(double lines)
   {
      config.innerLines = (int) lines;
      pushScreenDefinition();
   }

   /**
    * Sets the value of the INNER-LINES attribute which defines the number of text lines
    * in the widget.
    * 
    * @param    lines
    *           The new number of lines.
    */
   public void setInnerLines(NumberType lines)
   {
      if (lines == null || lines.isUnknown())
      {
         UnimplementedFeature.todo("INNER-LINES attribute unknown behavior is not known.");
         return;
      }
   
      setInnerLines(lines.intValue());
   }

   /**
    * Gets the value of the INNER-CHARS attribute which defines the number of character
    * columns in the widget.
    * 
    * @return   The number of character columns.
    */
   public integer getInnerChars()
   {
      return new integer(config.innerChars);
   }
   
   /**
    * Sets the value of the INNER-CHARS attribute which defines the number of character
    * columns in the widget.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setInnerChars(double chars)
   {
      config.innerChars = (int) chars;
      pushScreenDefinition();
   }
   
   /**
    * Sets the value of the INNER-CHARS attribute which defines the number of character
    * columns in the widget.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setInnerChars(NumberType chars)
   {
      if (chars == null || chars.isUnknown())
         return;
      
      setInnerChars(chars.intValue());
   }

   /**
    * Set value of the MAX-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setMaxChars(int chars)
   {
      config.maxChars = chars;
      pushScreenDefinition();
   }

   /**
    * Set value of the MAX-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setMaxChars(NumberType chars)
   {
      if (chars.isUnknown())
         return;
      
      setMaxChars(chars.intValue());
   }
   
   /**
    * Set value of the BUFFER-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setBufferChars(int chars)
   {
      config.bufferChars = chars;
      pushScreenDefinition();
   }
   
   /**
    * Set value of the BUFFER-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setBufferChars(NumberType chars)
   {
      if (chars.isUnknown())
         return;
      
      setBufferChars(chars.intValue());
   }
   
   /**
    * Set value of the BUFFER-LINES attribute.
    * 
    * @param   lines
    *          The new value for the attribute.
    */
   public void setBufferLines(int lines)
   {
      config.bufferLines = lines;
      pushScreenDefinition();
   }
   
   /**
    * Set value of the BUFFER-LINES attribute.
    * 
    * @param   lines
    *          The new value for the attribute.
    */
   public void setBufferLines(NumberType lines)
   {
      if (lines.isUnknown())
         return;
      
      setBufferLines(lines.intValue());
   }
   
   /**
    * Set value of the BOX attribute as opposite to passed value.
    * 
    * @param   box
    *          The new value for the attribute.
    */
   public void setNoBox(boolean box)
   {
      config.box = !box;
      pushScreenDefinition();
   }

   /**
    * Set value of the BOX attribute as opposite to passed value.
    * 
    * @param   box
    *          The new value for the attribute.
    */
   public void setNoBox(logical box)
   {
      if (box.isUnknown())
         return;
      
      setNoBox(box.booleanValue());
   }
   
   /**
    * Set value of the WORD-WRAP attribute as opposite to passed value.
    * 
    * @param   wordwrap
    *          The new value for the attribute.
    */
   public void setNoWordWrap(boolean wordwrap)
   {
      config.wordWrap = !wordwrap;
      pushScreenDefinition();
   }

   /**
    * Set value of the WORD-WRAP attribute as opposite to passed value.
    * 
    * @param   wordwrap
    *          The new value for the attribute.
    */
   public void setNoWordWrap(logical wordwrap)
   {
      if (wordwrap.isUnknown())
         return;
      
      setNoWordWrap(wordwrap.booleanValue());
   }
   
   /**
    * Set value of the SCROLLBAR-VERTICAL attribute.
    * 
    * @param   vertical
    *          The new value for the attribute.
    */
   public void setScrollbarVertical(boolean vertical)
   {
      config.scrollbarVertical = vertical;
      pushScreenDefinition();
   }

   /**
    * Set value of the SCROLLBAR-VERTICAL attribute.
    * 
    * @param   vertical
    *          The new value for the attribute.
    */
   public void setScrollbarVertical(logical vertical)
   {
      if (vertical == null || vertical.isUnknown())
      {
         UnimplementedFeature.todo("SCROLLBAR-VERTICAL attribute unknown behavior is not " +
                                   "known.");
         return;
      }
      
      setScrollbarVertical(vertical.booleanValue());
   }

   /**
    * Get value of the SCROLLBAR-VERTICAL attribute.
    *
    * @return  value of the attribute.
    */
   public logical isScrollbarVertical()
   {
      return new logical(config.scrollbarVertical);
   }
   
   /**
    * Get the value of the SCROLLBAR-HORIZONTAL attribute which controls whether the widget must
    * draw the horizontal scroll-bar if the inner width is more than the visible width.
    * 
    * @return   The value of the SCROLLBAR-HORIZONTAL attribute.
    */
   public logical isScrollbarHorizontal()
   {
      return new logical(config.scrollbarHorizontal);
   }
   
   /**
    * Set the value of the SCROLLBAR-HORIZONTAL attribute which controls whether the widget must
    * draw the horizontal scroll-bar if the inner width is more than the visible width.
    *
    * @param   horiz
    *          The new value for the attribute.
    */
   public void setScrollbarHorizontal(logical horiz)
   {
      if (horiz == null || horiz.isUnknown())
      {
         UnimplementedFeature.todo("SCROLLBAR-HORIZONTAL attribute unknown behavior is not " +
                                   "known.");
         return;
      }
      
      setScrollbarHorizontal(horiz.booleanValue());
   }
   
   /**
    * Set the value of the SCROLLBAR-HORIZONTAL attribute which controls whether the widget must
    * draw the horizontal scroll-bar if the inner width is more than the visible width.
    *
    * @param   horiz
    *          The new value for the attribute.
    */
   public void setScrollbarHorizontal(boolean horiz)
   {
      config.scrollbarHorizontal = horiz;
      pushScreenDefinition();
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public logical setSelection(int64 start, int64 end)
   {
      UnimplementedFeature.missing("SET-SELECTION method");
      return new logical();
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public logical setSelection(int64 start, int end)
   {
      UnimplementedFeature.missing("SET-SELECTION method");
      return new logical();
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public logical setSelection(int start, int64 end)
   {
      UnimplementedFeature.missing("SET-SELECTION method");
      return new logical();
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public logical setSelection(int start, int end)
   {
      UnimplementedFeature.missing("SET-SELECTION method");
      return new logical();
   }
   
   /**
    * Implements the DELETE-LINE() method which deletes the line of text in which the cursor is
    * positioned.
    *
    * @return   <code>true</code> if the line was deleted.
    */
   public logical deleteLine()
   {
      boolean result = LogicalTerminal.deleteEditorLine(getId());
      
      if (!result)
         UnimplementedFeature.todo("DELETE-LINE() return false compatibility testing is needed");
      
      return new logical(result);
   }
   
   /**
    * Implements the INSERT-STRING() method which inserts the given text into the editor at the
    * location at which the cursor is positioned.
    *
    * @param    txt
    *           The text to be inserted.
    *
    * @return   <code>true</code> if the text was inserted.
    */
   public logical insertString(character txt)
   {
      UnimplementedFeature.missing("INSERT-STRING() method");
      return new logical();
   }
   
   /**
    * Implements the INSERT-STRING() method which inserts the given text into the editor at the
    * location at which the cursor is positioned.
    *
    * @param    txt
    *           The text to be inserted.
    *
    * @return   <code>true</code> if the text was inserted.
    */
   public logical insertString(String txt)
   {
      UnimplementedFeature.missing("INSERT-STRING() method");
      return new logical();
   }
   
   /**
    * Implements the READ-FILE() method which replaces the editor's contents with those read
    * from the specified file.
    *
    * @param    name
    *           The relative or absolute file name.
    *
    * @return   <code>true</code> if the file was successfully read.
    */
   public logical readFile(character name)
   {
      if (name == null || name.isUnknown())
      {
         return new logical(false);
      }
      
      boolean res = LogicalTerminal.readFile(config.id.getId(), name.toStringMessage());
      
      return new logical(res);
   }
   
   /**
    * Implements the READ-FILE() method which replaces the editor's contents with those read
    * from the specified file.
    *
    * @param    name
    *           The relative or absolute file name.
    *
    * @return   <code>true</code> if the file was successfully read.
    */
   public logical readFile(String name)
   {
      return readFile(new character(name));
   }
   
   /**
    * Implements the REPLACE-SELECTION-TEXT() method which replaces the editor's currently
    * selected text with the given text.
    *
    * @param    txt
    *           The replacement text.
    *
    * @return   <code>true</code> if the text was successfully replaced.
    */
   public logical replaceSelectionText(character txt)
   {
      UnimplementedFeature.missing("REPLACE-SELECTION-TEXT() method");
      return new logical();
   }
   
   /**
    * Implements the REPLACE-SELECTION-TEXT() method which replaces the editor's currently
    * selected text with the given text.
    *
    * @param    txt
    *           The replacement text.
    *
    * @return   <code>true</code> if the text was successfully replaced.
    */
   public logical replaceSelectionText(String txt)
   {
      UnimplementedFeature.missing("REPLACE-SELECTION-TEXT() method");
      return new logical();
   }

   /**
    * Implements the SAVE-FILE() method which write the editor's contents to the specified file.
    *
    * @param    name
    *           The relative or absolute file name.
    *
    * @return   <code>true</code> if the file was successfully written.
    */
   @Override
   public logical saveFile(String name)
   {
      return saveFile(new character(name));
   }

   /**
    * Implements the SAVE-FILE() method which write the editor's contents to the specified file.
    *
    * @param    name
    *           The relative or absolute file name.
    *
    * @return   <code>true</code> if the file was successfully written.
    */
   @Override
   public logical saveFile(character name)
   {
      if (name == null || name.isUnknown())
      {
         return new logical(false);
      }

      boolean res = LogicalTerminal.writeFile(config.id.getId(), name.toStringMessage());

      return new logical(res);
   }

   /**
    * Get the value of the CURSOR-CHAR attribute which is the current cursor position as a
    * 1-based index on the current line.
    *
    * @return   The current position of the cursor on the current line.
    */
   public integer getCursorChar()
   {
      return new integer(config.cursorX + config.scrollX + 1);
   }
   
   /**
    * Set the value of the CURSOR-CHAR attribute which moves the cursor position to the
    * specified 1-based index on the current line.
    *
    * @param    idx
    *           The 1-based position to which to move.
    */
   public void setCursorChar(double idx)
   {
      if (idx < 1)
         UnimplementedFeature.todo("CURSOR-CHAR attribute < 1 behavior is not known.");
      
      UnimplementedFeature.todo("CURSOR-CHAR attribute compatibility testing is needed.");
      EditorHelper.setCursorX(config, (int) idx - 1);
      
      pushScreenDefinition();
   }

   /**
    * Set the value of the CURSOR-CHAR attribute which moves the cursor position to the
    * specified 1-based index on the current line.
    *
    * @param    idx
    *           The 1-based position to which to move.
    */
   public void setCursorChar(NumberType idx)
   {
      if (idx == null || idx.isUnknown())
      {
         UnimplementedFeature.todo("CURSOR-CHAR attribute unknown behavior is not known.");
         return;
      }
      
      setCursorChar(idx.doubleValue()); 
   }

   /**
    * Get the value of the CURSOR-LINE attribute which is the current cursor position as a
    * 1-based line number.
    *
    * @return   The current position of the cursor on the current line.
    */
   public integer getCursorLine()
   {
      return new integer(config.cursorY + config.scrollY + 1);
   }
   
   /**
    * Set the value of the CURSOR-LINE attribute which moves the cursor position to the
    * specified 1-based line number.
    *
    * @param    line
    *           The line to which to move.
    */
   public void setCursorLine(double line)
   {
      if (line < 1)
         UnimplementedFeature.todo("CURSOR-LINE attribute < 1 behavior is not known");
      
      UnimplementedFeature.todo("CURSOR-LINE attribute compatibility testing is needed.");
      
      // convert it to 0-based
      line -= 1;
      
      // we really need to figure out the way that the 4GL splits the scroll and cursor values
      // to make this compatible, this is just a first pass
      if (line < config.innerLines)
      {
         config.cursorY = (int) line;
         config.scrollY = 0;
      }
      else
      {
         config.cursorY = ((int) line) % config.innerLines;
         config.scrollY = ((int) line) - config.cursorY;
      }
      
      pushScreenDefinition();
   }

   /**
    * Set the value of the CURSOR-LINE attribute which moves the cursor position to the
    * specified 1-based line number.
    *
    * @param    line
    *           The line to which to move.
    */
   public void setCursorLine(NumberType line)
   {
      if (line == null || line.isUnknown())
      {
         UnimplementedFeature.todo("CURSOR-LINE attribute unknown behavior is not known.");
         return;
      }
      
      setCursorLine(line.doubleValue()); 
   }
   
   /**
    * Get the value of the NUM-LINES attribute which is the number of lines of text in the
    * editor.
    *
    * @return   The number of lines in the editor.
    */
   public integer getNumLines()
   {
      return new integer(config.numLines);
   }
}
